(* GeoProver test file for Mathematica, created on Jan 20 2003 *)
Needs["GeoProver`","GeoProver.m"]
<<"supp.m"
Off[General::spell1]
$PrePrint=InputForm
(* Example Arnon

The problem:
Let $ABCD$ be a square and $P$ a point on the line parallel to $BD$
through $C$ such that $l(BD)=l(BP)$, where $l(BD)$ denotes the
distance between $B$ and $D$. Let $Q$ be the intersection point of
$BF$ and $CD$. Show that $l(DP)=l(DQ)$.

The solution: *)

$vars=List[x1, x2, x3]
(* Points *)
$A=point[0,0]
$B=point[1,0]
$P=point[x1,x2]
(* coordinates *)
$D=rotate[$A,$B,1/2]
$C=parpoint[$D,$A,$B]
$Q=varpoint[$D,$C,x3]
(* polynomials *)
$polys=List[online[$P,parline[$C,ppline[$B,$D]]],
  eqdist[$B,$D,$B,$P], online[$Q,ppline[$B,$P]]]
(* conclusion *)
$con=eqdist[$D,$P,$D,$Q]
(* solution *)
$gb=geogbasis[$polys,$vars]
$result=geonormalf[$con,$gb,$vars]
(* Example CircumCenter_1

The problem:
The intersection point of the midpoint perpendiculars is the
center of the circumscribed circle.

The solution: *)

$parameters=List[a1, a2, b1, b2, c1, c2]
(* Points *)
$A=point[a1,a2]
$B=point[b1,b2]
$C=point[c1,c2]
(* coordinates *)
$M=intersectionpoint[pbisector[$A,$B],
  pbisector[$B,$C]]
(* conclusion *)
$result=List[ eqdist[$M,$A,$M,$B], eqdist[$M,$A,$M,$C] ]
(* Example EulerLine_1

The problem:
Euler's line: The center $M$ of the circumscribed circle,
the orthocenter $H$ and the barycenter $S$ are collinear and $S$
divides $MH$ with ratio 1:2.

The solution: *)

$parameters=List[a1, a2, b1, b2, c1, c2]
(* Points *)
$A=point[a1,a2]
$B=point[b1,b2]
$C=point[c1,c2]
(* coordinates *)
$S=intersectionpoint[median[$A,$B,$C],median[$B,$C,$A]]
$M=intersectionpoint[pbisector[$A,$B],
  pbisector[$B,$C]]
$H=intersectionpoint[altitude[$A,$B,$C],altitude[$B,$C,$A]]
(* conclusion *)
$result=List[iscollinear[$M,$H,$S], sqrdist[$S,fixedpoint[$M,$H,1/3]]]
(* Example Brocard_3

The problem:
Theorem about the Brocard points:
Let $\Delta\,ABC$ be a triangle. The circles $c_1$ through $A,B$ and
tangent to $g(AC)$, $c_2$ through $B,C$ and tangent to $g(AB)$, and
$c_3$ through $A,C$ and tangent to $g(BC)$ pass through a common
point.

The solution: *)

$parameters=List[u1, u2]
(* Points *)
$A=point[0,0]
$B=point[1,0]
$C=point[u1,u2]
(* coordinates *)
$M1=intersectionpoint[altitude[$A,$A,$C],pbisector[$A,$B]]
$M2=intersectionpoint[altitude[$B,$B,$A],pbisector[$B,$C]]
$M3=intersectionpoint[altitude[$C,$C,$B],pbisector[$A,$C]]
$c1=pccircle[$M1,$A]
$c2=pccircle[$M2,$B]
$c3=pccircle[$M3,$C]
$P=otherccpoint[$B,$c1,$c2]
(* conclusion *)
$result= oncircle[$P,$c3]
(* Example Feuerbach_1

The problem:
Feuerbach's circle or nine-point circle: The midpoint $N$ of $MH$ is
the center of a circle that passes through nine special points, the
three pedal points of the altitudes, the midpoints of the sides of the
triangle and the midpoints of the upper parts of the three altitudes.

The solution: *)

$parameters=List[u1, u2, u3]
(* Points *)
$A=point[0,0]
$B=point[u1,0]
$C=point[u2,u3]
(* coordinates *)
$H=intersectionpoint[altitude[$A,$B,$C],altitude[$B,$C,$A]]
$D=intersectionpoint[ppline[$A,$B],ppline[$H,$C]]
$M=intersectionpoint[pbisector[$A,$B],
  pbisector[$B,$C]]
$N=midpoint[$M,$H]
(* conclusion *)
$result=List[ eqdist[$N,midpoint[$A,$B],$N,midpoint[$B,$C]],
  eqdist[$N,midpoint[$A,$B],$N,midpoint[$H,$C]],
  eqdist[$N,midpoint[$A,$B],$N,$D] ]
(* Example FeuerbachTangency_1

The problem:
For an arbitrary triangle $\Delta\,ABC$ Feuerbach's circle (nine-point
circle) is tangent to its 4 tangent circles.

The solution: *)

$vars=List[x1, x2]
$parameters=List[u1, u2]
(* Points *)
$A=point[0,0]
$B=point[2,0]
$C=point[u1,u2]
$P=point[x1,x2]
(* coordinates *)
$M=intersectionpoint[pbisector[$A,$B], pbisector[$B,$C]]
$H=intersectionpoint[altitude[$A,$B,$C],altitude[$B,$C,$A]]
$N=midpoint[$M,$H]
$c1=pccircle[$N,midpoint[$A,$B]]
$Q=pedalpoint[$P,ppline[$A,$B]]
(* polynomials *)
$polys=List[onbisector[$P,$A,$B,$C], onbisector[$P,$B,$C,$A]]
(* conclusion *)
$con=iscctangent[pccircle[$P,$Q],$c1]
(* solution *)
$gb=geogbasis[$polys,$vars]
$result=geonormalf[$con,$gb,$vars]
(* Example GeneralizedFermatPoint_1

The problem:
A generalized theorem about Napoleon triangles:
Let $\Delta\,ABC$ be an arbitrary triangle and $P,Q$ and $R$ the third
vertex of isosceles triangles with equal base angles erected
externally on the sides $BC, AC$ and $AB$ of the triangle. Then the
lines $g(AP), g(BQ)$ and $g(CR)$ pass through a common point.

The solution: *)

$vars=List[x1, x2, x3, x4, x5]
$parameters=List[u1, u2, u3]
(* Points *)
$A=point[0,0]
$B=point[2,0]
$C=point[u1,u2]
$P=point[x1,x2]
$Q=point[x3,x4]
$R=point[x5,u3]
(* polynomials *)
$polys=List[eqdist[$P,$B,$P,$C], 
  eqdist[$Q,$A,$Q,$C],  
  eqdist[$R,$A,$R,$B], 
  eqangle[$R,$A,$B,$P,$B,$C], 
  eqangle[$Q,$C,$A,$P,$B,$C]]
(* conclusion *)
$con=isconcurrent[ppline[$A,$P], ppline[$B,$Q], ppline[$C,$R]]
(* solution *)
$sol=geosolve[$polys,$vars]
$result=geoeval[$con,$sol]
(* Example TaylorCircle_1

The problem:
Let $\Delta\,ABC$ be an arbitrary triangle. Consider the three
altitude pedal points and the pedal points of the perpendiculars from
these points onto the the opposite sides of the triangle. Show that
these 6 points are on a common circle, the {\em Taylor circle}.

The solution: *)

$parameters=List[u1, u2, u3]
(* Points *)
$A=point[u1,0]
$B=point[u2,0]
$C=point[0,u3]
(* coordinates *)
$P=pedalpoint[$A,ppline[$B,$C]]
$Q=pedalpoint[$B,ppline[$A,$C]]
$R=pedalpoint[$C,ppline[$A,$B]]
$P1=pedalpoint[$P,ppline[$A,$B]]
$P2=pedalpoint[$P,ppline[$A,$C]]
$Q1=pedalpoint[$Q,ppline[$A,$B]]
$Q2=pedalpoint[$Q,ppline[$B,$C]]
$R1=pedalpoint[$R,ppline[$A,$C]]
$R2=pedalpoint[$R,ppline[$B,$C]]
(* conclusion *)
$result=List[ isconcyclic[$P1,$P2,$Q1,$Q2], 
  isconcyclic[$P1,$P2,$Q1,$R1],
  isconcyclic[$P1,$P2,$Q1,$R2]]
(* Example Miquel_1

The problem:
Miquels theorem: Let $\Delta\,ABC$ be a triangle. Fix arbitrary points
$P,Q,R$ on the sides $AB, BC, AC$. Then the three circles through each
vertex and the chosen points on adjacent sides pass through a common
point.

The solution: *)

$parameters=List[c1, c2, u1, u2, u3]
(* Points *)
$A=point[0,0]
$B=point[1,0]
$C=point[c1,c2]
(* coordinates *)
$P=varpoint[$A,$B,u1]
$Q=varpoint[$B,$C,u2]
$R=varpoint[$A,$C,u3]
$X=otherccpoint[$P,p3circle[$A,$P,$R],p3circle[$B,$P,$Q]]
(* conclusion *)
$result=oncircle[$X,p3circle[$C,$Q,$R]]
(* Example PappusPoint_1

The problem:
Let $A,B,C$ and $P,Q,R$ be two triples of collinear points. Then by
the Theorem of Pappus the intersection points $g(AQ)\wedge g(BP),
g(AR)\wedge g(CP)$ and $g(BR)\wedge g(CQ)$ are collinear. 

Permuting $P,Q,R$ we get six such {\em Pappus lines}.  Those
corresponding to even resp. odd permutations are concurrent.

The solution: *)

$parameters=List[u1, u2, u3, u4, u5, u6, u7, u8]
(* Points *)
$A=point[u1,0]
$B=point[u2,0]
$P=point[u4,u5]
$Q=point[u6,u7]
(* coordinates *)
$C=varpoint[$A,$B,u3]
$R=varpoint[$P,$Q,u8]
(* conclusion *)
$result=isconcurrent[pappusline[$A,$B,$C,$P,$Q,$R],
  pappusline[$A,$B,$C,$Q,$R,$P], 
  pappusline[$A,$B,$C,$R,$P,$Q]]
(* Example IMO/36_1

The problem:
Let $A,B,C,D$ be four distinct points on a line, in that order. The
circles with diameters $AC$ and $BD$ intersect at the points $X$ and
$Y$. The line $XY$ meets $BC$ at the point $Z$. Let $P$ be a point on
the line $XY$ different from $Z$. The line $CP$ intersects the circle
with diameter $AC$ at the points $C$ and $M$, and the line $BP$
intersects the circle with diameter $BD$ at the points $B$ and
$N$. Prove that the lines $AM, DN$ and $XY$ are concurrent.

The solution: *)

$vars=List[x1, x2, x3, x4, x5, x6]
$parameters=List[u1, u2, u3]
(* Points *)
$X=point[0,1]
$Y=point[0,-1]
$M=point[x1,x2]
$N=point[x3,x4]
(* coordinates *)
$P=varpoint[$X,$Y,u3]
$Z=midpoint[$X,$Y]
$l=pbisector[$X,$Y]
$B=lineslider[$l,u1]
$C=lineslider[$l,u2]
$A=lineslider[$l,x5]
$D=lineslider[$l,x6]
(* polynomials *)
$polys=List[isconcyclic[$X,$Y,$B,$N], isconcyclic[$X,$Y,$C,$M],
  isconcyclic[$X,$Y,$B,$D], isconcyclic[$X,$Y,$C,$A],
  iscollinear[$B,$P,$N], iscollinear[$C,$P,$M]]
(* constraints *)
$nondeg=List[x5-u2,x1-u2,x6-u1,x3-u1]
(* conclusion *)
$con=isconcurrent[ppline[$A,$M],ppline[$D,$N],ppline[$X,$Y]]
(* solution *)
$sol=geosolveconstrained[$polys,$vars,$nondeg]
$result=geoeval[$con,$sol]
(* Example IMO/43_2

The problem:

No verbal problem description available

The solution: *)

$vars=List[x1, x2]
$parameters=List[u1]
(* Points *)
$B=point[-1,0]
$C=point[1,0]
(* coordinates *)
$O=midpoint[$B,$C]
$gamma=pccircle[$O,$B]
$D=circleslider[$O,$B,u1]
$E=circleslider[$O,$B,x1]
$F=circleslider[$O,$B,x2]
$A=sympoint[$B,ppline[$O,$D]]
$J=intersectionpoint[ppline[$A,$C], parline[$O, ppline[$A,$D]]]
$m=pbisector[$O,$A]
$P1=pedalpoint[$J,$m]
$P2=pedalpoint[$J,ppline[$C,$E]]
$P3=pedalpoint[$J,ppline[$C,$F]]
(* polynomials *)
$polys=List[online[$E,$m], online[$F,$m]]
(* constraints *)
$nondeg=List[x1-x2]
(* conclusion *)
$con=List[eqdist[$J,$P1,$J,$P2], eqdist[$J,$P1,$J,$P3]]
(* solution *)
(* 

    geosolveconstrained does not finish this computation. Why ??

$sol=geosolveconstrained[$polys,$vars,$nondeg]
$result=geosimplify[geoeval[$con,$sol]]
*)
Quit[]

